##############################################################################
##############################################################################
#                               Commands.tcl
##############################################################################
##############################################################################
# Here I will implement Tcl equivalents of unix 'touch' and 'sed' commands,   
# or at least, as far as 'sed' is concerned, what little of 'sed' Getleft
# needs.
##############################################################################
##############################################################################
# Copyright 2000 Andrs Garca Garca  -- fandom@retemail.es
# Distributed under the terms of the GPL license v2
##############################################################################
##############################################################################

namespace eval Commands {

set sedIndex 0

##############################################################################
# Touch
#    This procedure will create an empty file
#
# Parameters:
#    fileName: The file to create.
##############################################################################
proc Touch {fileName} {

    if {![info exists "$fileName"]} {
        set handle [open "$fileName" w]
        close $handle
    }
    return
}

##############################################################################
# SedReadFile
#    Reads a given file into memory for the pseudosed command to work on.
#
# Parameters:
#    fileName: The file to read.
#
# Returns:
#    - '0' if all went well.
#    - '1' if not.
##############################################################################
proc SedReadFile {fileName} {
    variable workFile 

    catch {unset workFile}

    if {[catch {open "$fileName" r} handle]} {
        return 1
    }
    for {set i 0} {![eof $handle]} {incr i} {
        set workFile($i) [gets $handle]
    }
    close $handle

    return 0
}

##############################################################################
# SedWriteFile
#     Saves whatever is in the workFile array into the given file. The file
#     must not already exist.
#
# Parameter:
#     fileName: file to use to save the data.
#
# Returns:
#    - '0' if all went well.
#    - '1' if not.
##############################################################################
proc SedWriteFile {fileName} {
    variable workFile

    if {[catch {open "$fileName" w} handle]} {
        return 1
    }
    for {set i 0} {![catch "set workFile($i)"]} {incr i} {
        puts $handle "$workFile($i)"
    }
    close $handle

    return 0
}

##############################################################################
# SedChangeEnter
#     Enters a new changes in to the 'sedChanges' array.
#
# Parameters:
#    - old: The regular expresion to subtitute.
#    - new: The substitute.
##############################################################################
proc SedChangeEnter {old new} {
    variable sedChanges
    variable sedIndex

    regsub {^\.\.}    $old {\.\.} old
    regsub {^\.}      $old {\.}   old
    regsub -all {\*}  $old {\\*}  old
    regsub -all {\+}  $old {\\+}  old
    regsub -all {\?}  $old {\\?}  old
    regsub -all {&}   $new {\&}   new

    set sedChanges($sedIndex,old) $old
    set sedChanges($sedIndex,new) $new

    incr sedIndex

    return
}

##############################################################################
# SedChange
#     Goes through the file in 'workFile' chaging one link.
#
# Parameter:
#     index: The index of the link to change in the sedChanges variable.
#
# Returns:
#    - '0' if there was no change.
#    - '1' if a change was found.
##############################################################################
proc SedChange {index} {
    variable workFile
    variable sedChanges
    variable startLine

    set old $sedChanges($index,old)
    set new $sedChanges($index,new)

    for {set i $startLine} {![catch "set workFile($i)"]} {incr i} {
        if {[regsub -all "$old" $workFile($i) "$new" workFile($i)]} {
            set startLine $i
            return 1
        }
    }
    return 0
}

##############################################################################
# Sed
#     Goes through a given file and make the requested changes to it.
#
# Parameter:
#     fileName: file to change.
#     skip: whether to skip odd changes if the prior one is successful
#
# Returns:
#    - '0' if all went well.
#    - '1' if not.
##############################################################################
proc Sed {fileName {skip 0}} {
    variable workFile
    variable sedChanges
    variable sedIndex
    variable startLine

    if {[file exists $fileName.html]} {
        set fileName $fileName.html
    }

    if {[SedReadFile $fileName]==1} {return 1}

    for {set i 0 ; set startLine 0} {![catch "set sedChanges($i,old)"]} {incr i} {
        if {([SedChange $i]==1)&&([expr $i%2]==0)} {
            if {$skip==1} {
                incr i    
            }
        }
    }

    catch {unset sedChanges}
    set sedIndex 0

    if {[catch {file rename "$fileName" "$fileName.orig"} result]} {
        file delete $fileName
    }
    if {[SedWriteFile $fileName]==1} {return 1}

    return 0
}

}
